%% Simulation einer QPSK-bertragungsstrecke mit T/2-Entzerrer 
% C. Roppel, FH Schmalkalden
% 31.08.2010
% 
% Erluterungen siehe "Versuche mit Matlab",
% Begleitmaterial zum Buch "Grundlagen der digitalen Kommunikationstechnik"
% Fachbuchverlag Leipzig, 2006
%
% Getestet mit Matlab 7.4.0 und 7.9.0
%
clear
clc
close all
format short

%% Modulationsparameter fr QPSK
m = 4;          % m-wertige Symbole
k = log2(m);    % k Bit pro Symbol
alpha = 0.5;    % Roll-off-Faktor
nsym = 5e5;     % Anzahl der Symbole
os = 8;         % Samples pro Symbol (oversampling ratio)
dly_rc = 3;     % Verzgerung des Wurzel-Kosinus-roll-off-Filters

% Modulator und Demodulator
modobj = modem.pskmod('M', m, 'PhaseOffset', pi/4, 'SymbolOrder', 'Gray')
demodobj = modem.pskdemod('M', m, 'PhaseOffset', pi/4, 'SymbolOrder', 'Gray')

% Tiefpassfilter
B_K = (1 + alpha)/2;                 % Bandbreite des Basisbandsignals
fir_lp = fir1(256, B_K/(os/2));      % TP der Bandbreite B_K
dly_lp = (length(fir_lp)-1)/(2*os);  % Verzgerung des TP-Filters

%% Bit- und Symbolfolge
dbit = randint(nsym*k, 1, [0 1], 12345);
dsym = bi2de(reshape(dbit, k, nsym).', 'left-msb');

%% Kanalmodell
% mehrwege = 0: verzerrungsfreier Kanal
% mehrwege = 1: Mehrwegekanal
mehrwege = 0;
dt = 2;         % Laufzeitdifferenz (in Samples)
a = 0.8;        % Dmpfung

% Impulsantwort des Kanals
if mehrwege == 0
    hch = 1; 
else
    hch = zeros(1, dt+1);
    hch(1) = 1; 
    hch(dt+1) = a;
end

%% T/2-Entzerrer
Ne = 33;                      % Anzahl der Koeffizienten
dly_eq = round((Ne-1)/(2*2)); % Verzgerung des Entzerrers

% Zu entzerrender Impuls g(n)
rcos = rcosine(1, os, 'fir/sqrt', alpha, dly_rc);
gn = conv(rcos, hch);
gn = downsample(gn, os/2, 0);

% Rauschleistung
var_n = 0.1;
% Faltungsmatrix mit m + 1 + N Zeilen und m + 1 Spalten
gn = gn(:);
F = toeplitz([gn; zeros(Ne-1, 1)], [gn(1) zeros(1, Ne-1)]);
% F hat z Zeilen und s Spalten, Weglassen jeder zweiten Zeile
[z, s] = size(F);
F(2:2:z, :) = [];
% Ideales Ausgangssignal
[z, s] = size(F);
yid = zeros(z, 1);
yid(round(z/2) + 1) = 1;
% Berechnung der Koeffizienten
I = diag(ones(1, s));
en = (F'*F + var_n*I)\F'*yid;

%% Modulation - Kanal - Empfnger
% Modulation
xmod = modulate(modobj, dsym);

% Pulsformfilter (Wurzel-Kosinus-roll-off-Filter)
x = rcosflt(xmod, 1, os, 'fir/sqrt', alpha, dly_rc);
x = x((os*dly_rc + 1):(end - os*dly_rc));
eyediagram(x(os*dly_rc:10000), 2*os, 2, 1);
figure; periodogram(real(x), [], 4096, os);

% Faltung mit der Kanalimpulsantwort
y = filter(hch, 1, x);
figure; periodogram(real(y), [], 4096, os);

% Augendiagramm nach dem Entzerrer, kein Rauschen
yeq = downsample(y, os/2);
yeq = filter(en, 1, yeq);
eyediagram(yeq(os*dly_rc:10000), 4, 2, 1);

% Schleife fr verschiedene Eb/n0
Ebn0List = [6:10];                        % Eb/n0 in dB
berTheoList = zeros(1, length(Ebn0List)); % theoretsche Werte
berList = zeros(1, length(Ebn0List));     % aus Simulation bestimmte Werte

for i = 1:length(Ebn0List)    
    % AWGN-Kanal
    Ebn0 = Ebn0List(i);
    snr = Ebn0 + 10*log10(k) - 10*log10(os);
    r = awgn(y, snr, 'measured', 12345);
    % Empfnger: TP-Filter, Entzerrer, Demodulation  
    r = filter(fir_lp, 1, r);
    r = downsample(r, os/2);
    req = filter(en, 1, r);
    req = downsample(req, 2);    
    ysym = demodulate(demodobj, req);
    % Bestimmung der Bitfehlerhufigkeit
    ybit = de2bi(ysym, 'left-msb');
    ybit = reshape(ybit.', prod(size(ybit)), 1);
    % Kompensation der Verzgerung der Filter
    dbit_ber = dbit(1:(end - 2*(dly_lp + dly_eq + 1)));
    ybit_ber = ybit((2*(dly_lp + dly_eq + 1) + 1):end);
    [nerror, ber]= biterr(dbit_ber, ybit_ber);
    berList(i) = ber;   
    % Theoretische Werte 
    berTheoList(i) = 0.5*erfc(sqrt(10^(Ebn0/10)));
    fprintf('i: %d  Bitfehler: %d  Bitfehlerhufigkeit: %f\n', i, nerror, ber);
end
scatterplot(req(1:1e4));

%% Plot BER vs. Eb/n0
figure; semilogy(Ebn0List, berList, '-.or'); grid; title('QPSK Equalizer');
xlabel('E_b/n_0 in dB'); ylabel('Bitfehlerhufigkeit'); hold on;
plot(Ebn0List, berTheoList,'-x'); hold off;
legend('Simulation', 'Theoretisch', 'Location','SouthWest');
